@@ -420,6 +420,9 @@ class FormatTest(unittest.TestCase):
         self.assertEqual(format(1+2j, "\u2007^8"), "\u2007(1+2j)\u2007")
         self.assertEqual(format(0j, "\u2007^4"), "\u20070j\u2007")
 
+    # TODO: RUSTPYTHON formatting does not support locales
+    # See https://github.com/RustPython/RustPython/issues/5181
+    @unittest.skip("formatting does not support locales")
     def test_locale(self):
         try:
             oldloc = locale.setlocale(locale.LC_ALL)
